@@ -15,7 +15,10 @@ YELLOW = 'rgb(221,213,51)'
 YELLOW_NUM = (221,213,51)
 
 LIME = 'rgb(220, 243, 31)'
-LIME_NUM = (220, 243, 31)
+LIME_NUM = (128, 255, 128)
+
+PINK = 'rgb(232, 124, 243)'
+PINK_NUM = (255, 124, 243)
 
 WHITE = 'rgb(255,255,255)'
 WHITE_NUM = (255,255,255)
@@ -26,3 +29,5 @@ BLACK_NUM = (0,0,0)
 GREY = 'rgb(223, 210, 213)'
 GREY_NUM = (192, 196, 199)
 TRACE_COLORS = [TEAL_NUM, ORANGE_NUM, GREEN_NUM]
+
+MARKER_HOVER = PINK_NUM
\ No newline at end of file
