@@ -1,5 +1,4 @@
 import logging
-import glob
 
 from PySide import QtCore
 import numpy as np  # FIXME: move sweep playback out of here
@@ -40,11 +39,12 @@ class SpecAController(QtCore.QObject):
     state_change = QtCore.Signal(SpecAState, list)
     capture_receive = QtCore.Signal(SpecAState, float, float, object, object, object, object)
     options_change = QtCore.Signal(dict, list)
-
+    plot_change = QtCore.Signal(dict, list)
     def __init__(self):
         super(SpecAController, self).__init__()
         self._dsp_options = {}
         self._options = {}
+        self._plot_options = {}
 
 
     def set_device(self, dut=None, playback_filename=None):
@@ -71,7 +71,7 @@ class SpecAController(QtCore.QObject):
             state_json = vrt_packet.fields['speca']
             # support old playback files
             if state_json['device_identifier'] == 'unknown':
-                state_json['device_identifier'] = 'ThinkRF,WSA5000 v3,,'
+                state_json['device_identifier'] = 'ThinkRF,WSA5000 v3,0,0'
             dut = Playback(state_json['device_class'],
                 state_json['device_identifier'])
             self._sweep_device = SweepDevice(dut)
@@ -92,24 +92,14 @@ class SpecAController(QtCore.QObject):
         self._apply_complete_settings(state_json, bool(self._playback_file))
         self.start_capture()
 
-    def start_recording(self, filename=None):
+    def start_recording(self, filename):
         """
-        Start a new recording with filename, or autogenerated filename.
-        Stops previous recording if one in progress. Does nothing if we're
+        Start a new recording. Does nothing if we're
         currently playing a recording.
         """
         if self._playback_file:
             return
         self.stop_recording()
-        if not filename:
-            names = glob.glob('recording-*.vrt')
-            last_index = -1
-            for n in names:
-                try:
-                    last_index = max(last_index, int(n[10:-4]))
-                except ValueError:
-                    pass
-            filename = 'recording-%04d.vrt' % (last_index + 1)
         self._recording_file = open(filename, 'wb')
         self._dut.set_recording_output(self._recording_file)
         self._dut.inject_recording_state(self._state.to_json_object())
@@ -157,6 +147,7 @@ class SpecAController(QtCore.QObject):
         device_set.pop('pll_reference')
         device_set.pop('iq_output_path')
         device_set.pop('trigger')
+
         self._sweep_device.capture_power_spectrum(
             self._state.center - self._state.span / 2.0,
             self._state.center + self._state.span / 2.0,
@@ -366,15 +357,16 @@ class SpecAController(QtCore.QObject):
                 pow_data, usable_bins, fstart, fstop = (
                     trim_to_usable_fstart_fstop(
                         pow_data, usable_bins, fstart, fstop))
-
-            self.capture_receive.emit(
-                self._state,
-                fstart,
-                fstop,
-                data['data_pkt'],
-                pow_data,
-                usable_bins,
-                None)
+            #FIXME: Find out why there is a case where pow_data may be empty
+            if pow_data.any():
+                self.capture_receive.emit(
+                    self._state,
+                    fstart,
+                    fstop,
+                    data['data_pkt'],
+                    pow_data,
+                    usable_bins,
+                    None)
 
     def process_sweep(self, fstart, fstop, data):
         sweep_segments = list(self._sweep_device.sweep_segments)
@@ -404,6 +396,11 @@ class SpecAController(QtCore.QObject):
         Emit signal and handle special cases where extra work is needed in
         response to a state change.
         """
+
+        # make sure resolution of center are the same as the device's tunning resolution
+        center = float(np.round(state.center, -1 * int(np.log10(self._dut.properties.TUNING_RESOLUTION))))
+        state = SpecAState(state, center=center)
+
         if not state.sweeping():
             # force span to correct value for the mode given
             if state.decimation > 1:
@@ -416,7 +413,17 @@ class SpecAController(QtCore.QObject):
             if not self._state or span != self._state.span:
                 changed.append('span')
 
+        if 'mode' in changed:
+            # check if RBW is appropriate for given mode
+            if state.rbw not in self._dut.properties.RBW_VALUES[state.rfe_mode()]:
+                if state.sweeping():
+                    rbw = self._dut.properties.RBW_VALUES[state.rfe_mode()][0]
+                    state = SpecAState(state, rbw=rbw)
+                else:
+                    rbw = self._dut.properties.RBW_VALUES[state.rfe_mode()][-1]
+                    state = SpecAState(state, rbw=rbw)
         self._state = state
+
         # start capture loop again when user switches output path
         # back to the internal digitizer XXX: very WSA5000-specific
         if 'device_settings.iq_output_path' in changed:
@@ -451,10 +458,10 @@ class SpecAController(QtCore.QObject):
 
         :param kwargs: keyword arguments of SpecAState attributes
         """
-
         if self._state is None:
             logger.warn('apply_settings with _state == None: %r' % kwargs)
             return
+
         state = SpecAState(self._state, **kwargs)
         self._state_changed(state, kwargs.keys())
 
@@ -505,6 +512,16 @@ class SpecAController(QtCore.QObject):
             self.enable_user_xrange_control(
                 not kwargs['free_plot_adjustment'])
 
+    def apply_plot_options(self, **kwargs):
+        """
+        Apply plot option changes and signal the change
+
+        :param kwargs: keyword arguments of the plot options
+        """
+        self._plot_options.update(kwargs)
+        self.plot_change.emit(dict(self._plot_options),
+            kwargs.keys())
+
     def get_options(self):
         return dict(self._options)
 
