@@ -2,7 +2,7 @@ from PySide import QtGui
 
 from pyrf.units import M
 from pyrf.gui import colors
-from pyrf.gui.widgets import QDoubleSpinBoxPlayback
+from pyrf.gui.widgets import QComboBoxPlayback, QDoubleSpinBoxPlayback
 from pyrf.gui.fonts import GROUP_BOX_FONT
 
 class FrequencyControls(QtGui.QGroupBox):
@@ -38,6 +38,10 @@ class FrequencyControls(QtGui.QGroupBox):
         grid.addWidget(freq_inc_label, 2, 0, 1, 1)
         grid.addWidget(freq_inc_steps, 2, 1, 1, 1)
 
+        rbw_label, rbw_combo = self._rbw_controls()
+        grid.addWidget(rbw_label, 2, 3, 1, 1)
+        grid.addWidget(rbw_combo, 2, 4, 1, 1)
+
         grid.setColumnStretch(0, 4)
         grid.setColumnStretch(1, 9)
         grid.setColumnStretch(2, 1)
@@ -53,6 +57,17 @@ class FrequencyControls(QtGui.QGroupBox):
 
     def state_changed(self, state, changed):
         self.gui_state = state
+
+        def enable_disable_edit_boxes():
+            if state.sweeping():
+                self._fstart_edit.setEnabled(not state.playback)
+                self._fstop_edit.setEnabled(not state.playback)
+                self._bw_edit.setEnabled(not state.playback)
+            else:
+                self._fstart_edit.setEnabled(False)
+                self._fstop_edit.setEnabled(False)
+                self._bw_edit.setEnabled(False)
+
         if 'mode' in changed:
             min_tunable = float(self.dut_prop.MIN_TUNABLE[state.rfe_mode()])
             max_tunable = float(self.dut_prop.MAX_TUNABLE[state.rfe_mode()])
@@ -68,26 +83,28 @@ class FrequencyControls(QtGui.QGroupBox):
             self._bw_edit.quiet_update(
                 tuning_res / M, (max_tunable - min_tunable) / M)
 
-            if state.sweeping():
-                self._fstart_edit.setEnabled(not state.playback)
-                self._fstop_edit.setEnabled(not state.playback)
-                self._bw_edit.setEnabled(not state.playback)
-            else:
-                self._fstart_edit.setEnabled(False)
-                self._fstop_edit.setEnabled(False)
-                self._bw_edit.setEnabled(False)
-
+            enable_disable_edit_boxes()
+            self._update_rbw_options()
         if any(x in changed for x in ('center', 'span', 'decimation', 'mode')):
             self._update_freq_edit()
 
         if 'playback' in changed:
             self._freq_edit.setEnabled(not state.playback)
+            if state.playback:
+                self._rbw_box.playback_value(str(state.rbw))
+            else:
+                self._rbw_box.setEnabled(True)
+            enable_disable_edit_boxes()
+            self._update_rbw_options()
 
         if 'device_settings.iq_output_path' in changed:
             if state.device_settings['iq_output_path'] == 'CONNECTOR':
                 self._fstart_edit.setEnabled(False)
                 self._fstop_edit.setEnabled(False)
                 self._bw_edit.setEnabled(False)
+                self._rbw_box.setEnabled(False)
+            elif state.device_settings['iq_output_path'] == 'DIGITIZER':
+                self._rbw_box.setEnabled(True)
 
     def _freq_incr(self):
         steps_label = QtGui.QLabel("Adjust:")
@@ -124,7 +141,7 @@ class FrequencyControls(QtGui.QGroupBox):
             self.controller.apply_settings(center=freq_edit.value() * M)
             if self.gui_state.device_settings['iq_output_path'] == 'CONNECTOR':
                 self.controller.apply_device_settings(freq = freq_edit.value() * M)
-        freq_edit.valueChanged.connect(freq_change)
+        freq_edit.editingFinished.connect(freq_change)
         return cfreq, freq_edit
 
     def _bw_controls(self):
@@ -134,7 +151,7 @@ class FrequencyControls(QtGui.QGroupBox):
         bw_edit.setSuffix(' MHz')
         def freq_change():
             self.controller.apply_settings(span=bw_edit.value() * M)
-        bw_edit.valueChanged.connect(freq_change)
+        bw_edit.editingFinished.connect(freq_change)
         self._bw_edit = bw_edit
         return bw, bw_edit
 
@@ -151,7 +168,7 @@ class FrequencyControls(QtGui.QGroupBox):
                 center = (fstop + fstart) / 2.0,
                 span = (fstop - fstart),
                 )
-        freq.valueChanged.connect(freq_change)
+        freq.editingFinished.connect(freq_change)
         self._fstart_edit = freq
         return fstart, freq
 
@@ -168,10 +185,22 @@ class FrequencyControls(QtGui.QGroupBox):
                 center = (fstop + fstart) / 2.0,
                 span = (fstop - fstart),
                 )
-        freq.valueChanged.connect(freq_change)
+        freq.editingFinished.connect(freq_change)
         self._fstop_edit = freq
         return fstop, freq
 
+    def _rbw_controls(self):
+        rbw_label = QtGui.QLabel('RBW:')
+        rbw_box = QComboBoxPlayback()
+        rbw_box.setToolTip("Change the RBW of the FFT plot")
+        def new_rbw():
+            self.controller.apply_settings(rbw=self._rbw_values[
+                rbw_box.currentIndex()])
+        rbw_box.currentIndexChanged.connect(new_rbw)
+        self._rbw_box = rbw_box
+        return rbw_label, rbw_box
+
+
     def _update_freq_edit(self):
         """
         update the spin boxes from self.gui_state
@@ -205,3 +234,30 @@ class FrequencyControls(QtGui.QGroupBox):
         self._fstop.setEnabled(False)
         self._fstop_edit.setEnabled(False)
 
+    def _rbw_replace_items(self, items):
+        for i in range(self._rbw_box.count()):
+            self._rbw_box.removeItem(0)
+        self._rbw_box.addItems(items)
+
+    def _update_rbw_options(self):
+        """
+        populate RBW drop-down with reasonable values for the current mode
+        """
+        if hasattr(self, 'gui_state'):
+            mode = self.gui_state.rfe_mode()
+
+            self._rbw_values = self.dut_prop.RBW_VALUES[mode]
+            if mode == 'HDR':
+                unit = 'Hz'
+                div = 1
+            else:
+                unit = 'KHz'
+                div = 1000
+            self._rbw_box.quiet_update(
+                ["%0.2f " % (float(p) / div) + unit for p in self._rbw_values])
+
+            if self.gui_state.sweeping():
+                self._rbw_box.setCurrentIndex(0)
+
+            else:
+                self._rbw_box.setCurrentIndex(self._rbw_box.count() - 1)
