@@ -12,15 +12,16 @@ and placed to left of the controls.
 from PySide import QtGui, QtCore
 import numpy as np
 import math
+import glob
 
-from pkg_resources import parse_version
+from pkg_resources import parse_version, require
 
 from pyrf.gui import colors
 from pyrf.gui import fonts
 from pyrf.gui import labels
 from pyrf.gui import gui_config
 from pyrf.gui.controller import SpecAController
-
+from pyrf.version import __version__
 from pyrf.connectors.twisted_async import TwistedConnector
 from pyrf.config import TriggerSettings, TRIGGER_TYPE_LEVEL
 from pyrf.units import M
@@ -38,21 +39,18 @@ from pyrf.gui.trace_controls import TraceControls
 
 VIEW_OPTIONS = [
     ('&IQ Plots', 'iq_plots', False),
-    ('&Waterfall Plot', 'waterfall_plot', False),
-    ]
-
-DSP_OPTIONS = [
-    ('&IQ Offset Correction', 'dsp.correct_phase', True),
-    ('&DC Offset', 'dsp.hide_differential_dc_offset', True),
-    ('&Convert to dBm', 'dsp.convert_to_dbm', True),
-    ('Apply &Spectral Inversion', 'dsp.apply_spec_inv', True),
-    ('Apply &Hanning Window', 'dsp.apply_window', True),
+    ('&Spectrogram Plot', 'waterfall_plot', False),
+    ('&Persistence Plot', 'persistence_plot', False),
     ]
 
 DEVELOPER_OPTIONS = [
     ('Show &Attenuated Edges', 'show_attenuated_edges', False),
     ('Show &Sweep Steps', 'show_sweep_steps', False),
     ('&Free Plot Adjustment', 'free_plot_adjustment', False),
+    ('&IQ Offset Correction', 'dsp.correct_phase', True),
+    ('&DC Offset', 'dsp.hide_differential_dc_offset', True),
+    ('Apply &Spectral Inversion', 'dsp.apply_spec_inv', True),
+    ('Apply &Hanning Window', 'dsp.apply_window', True),
     ]
 
 # FIXME: we shouldn't be calculating fft in this module
@@ -77,7 +75,8 @@ class MainWindow(QtGui.QMainWindow):
     """
     The main window and menus
     """
-    def __init__(self, dut_address=None, playback_filename=None):
+    def __init__(self, dut_address=None, playback_filename=None,
+            developer_menu=False):
         super(MainWindow, self).__init__()
         screen = QtGui.QDesktopWidget().screenGeometry()
         WINDOW_WIDTH = max(screen.width() * 0.7, MINIMUM_WIDTH)
@@ -85,13 +84,13 @@ class MainWindow(QtGui.QMainWindow):
         self.resize(WINDOW_WIDTH,WINDOW_HEIGHT)
 
         self.controller = SpecAController()
-        self.init_menu_bar()
+        self.init_menu_bar(developer_menu)
         self.initUI(dut_address, playback_filename)
 
     def initUI(self, dut_address, playback_filename):
         self.mainPanel = MainPanel(self.controller, self)
 
-        self.setWindowTitle('PyRF RTSA')
+        self.setWindowTitle('PyRF RTSA: ' + __version__)
         self.setCentralWidget(self.mainPanel)
         if dut_address:
             self.open_device(dut_address, True)
@@ -100,7 +99,8 @@ class MainWindow(QtGui.QMainWindow):
         else:
             self.open_device_dialog()
 
-    def init_menu_bar(self):
+    def init_menu_bar(self, developer_menu=False):
+
         open_action = QtGui.QAction('&Open Device', self)
         open_action.triggered.connect(self.open_device_dialog)
         play_action = QtGui.QAction('&Playback Recording', self)
@@ -138,23 +138,33 @@ class MainWindow(QtGui.QMainWindow):
             self.view_menu.addAction(checkbox_action(
                 self.controller.apply_options, text, option, default))
 
-        self.dsp_menu = menubar.addMenu('&DSP Options')
-        for text, option, default in DSP_OPTIONS:
-            self.dsp_menu.addAction(checkbox_action(
-                self.controller.apply_options, text, option, default))
-
-        self.developer_menu = menubar.addMenu('D&eveloper Options')
-        for text, option, default in DEVELOPER_OPTIONS:
-            self.developer_menu.addAction(checkbox_action(
-                self.controller.apply_options, text, option, default))
+        if developer_menu:
+            self.developer_menu = menubar.addMenu('D&eveloper Options')
+            for text, option, default in DEVELOPER_OPTIONS:
+                self.developer_menu.addAction(checkbox_action(
+                    self.controller.apply_options, text, option, default))
 
         self.controller.apply_options(
             **dict((option, default) for text, option, default
-            in VIEW_OPTIONS + DSP_OPTIONS + DEVELOPER_OPTIONS))
+            in VIEW_OPTIONS + DEVELOPER_OPTIONS))
 
     def start_recording(self):
         self.stop_action.setDisabled(False)
-        self.controller.start_recording()
+        names = glob.glob('recording-*.vrt')
+        last_index = -1
+        for n in names:
+            try:
+                last_index = max(last_index, int(n[10:-4]))
+            except ValueError:
+                pass
+        filename = 'recording-%04d.vrt' % (last_index + 1)
+        record_filename, file_type = QtGui.QFileDialog.getSaveFileName(self,
+            "Create Recording",
+            filename,
+            "VRT Packet Capture Files (*.vrt)",
+            )
+        if record_filename:
+            self.controller.start_recording(record_filename)
 
     def stop_recording(self):
         self.stop_action.setDisabled(True)
@@ -188,7 +198,7 @@ class MainWindow(QtGui.QMainWindow):
         self.show()
         dut = WSA(connector=TwistedConnector(self._get_reactor()))
         yield dut.connect(name)
-        self.setWindowTitle('PyRF RTSA Connected To: %s' %name)
+        self.setWindowTitle('PyRF RTSA %s Connected To: %s' % (__version__ , name))
         if hasattr(dut.properties, 'MINIMUM_FW_VERSION') and parse_version(
                 dut.fw_version) < parse_version(dut.properties.MINIMUM_FW_VERSION):
             too_old = QtGui.QMessageBox()
@@ -246,7 +256,7 @@ class MainPanel(QtGui.QWidget):
         self._waterfall_range = None, None, None
 
         self.options_changed(controller.get_options(),
-            ['iq_plots', 'waterfall_plot'])
+            ['iq_plots', 'waterfall_plot', 'persistence_plot'])
 
     def device_changed(self, dut):
         self.plot_state = gui_config.PlotState(dut.properties)
@@ -262,11 +272,10 @@ class MainPanel(QtGui.QWidget):
         :param changed: list of attribute names changed
         """
         self.gui_state = state
-
         if 'mode' in changed:
             rfe_mode = state.rfe_mode()
             self._update_iq_plot_visibility()
-
+            self.update_rbw_label()
             if rfe_mode in ('DD', 'IQIN'):
                 freq = self.dut_prop.MIN_TUNABLE[rfe_mode]
                 full_bw = self.dut_prop.FULL_BW[rfe_mode]
@@ -291,24 +300,23 @@ class MainPanel(QtGui.QWidget):
             if state.device_settings['iq_output_path'] == 'CONNECTOR':
                 # remove plots
                 self._plot_layout.hide()
+                self.hide_labels()
                 if self._main_window.isMaximized():
                     self._main_window.showNormal()
-
                 # resize window
                 for x in range(self.plot_width):
                     self._grid.setColumnMinimumWidth(x, 0)
                 screen = QtGui.QDesktopWidget().screenGeometry()
-
                 self.setMinimumWidth(0)
                 self.setMinimumHeight(0)
                 self._main_window.setMinimumWidth(0)
                 self._main_window.setMinimumHeight(0)
                 self.resize(0,0)
                 self._main_window.resize(0,0)
-
             else:
                 # show plots
                 self._plot_layout.show()
+                self.show_labels()
                 # resize window
                 for x in range(self.plot_width):
                     self._grid.setColumnMinimumWidth(x, 300)
@@ -316,75 +324,66 @@ class MainPanel(QtGui.QWidget):
                 self.setMinimumWidth(MINIMUM_WIDTH)
                 self.setMinimumHeight(MINIMUM_HEIGHT)
                 WINDOW_WIDTH = max(screen.width() * 0.7, MINIMUM_WIDTH)
-                WINDOW_HEIGHT = max(screen.height() * 0.6, MINIMUM_HEIGHT)
+                WINDOW_HEIGHT = max(screen.height() * 0.7, MINIMUM_HEIGHT)
                 self._main_window.resize(WINDOW_WIDTH,WINDOW_HEIGHT)
 
+        if 'rbw' in changed:
+            self.update_rbw_label()
 
-    def keyPressEvent(self, event):
-        if not self.dut_prop:
-            return
+        if 'span' in changed:
+            self.update_span_label()
 
-        hotkey_dict = {
-            'M': self.trace_group._marker_control,
-            'P': self.trace_group._find_peak,
-            }
+    def show_labels(self):
+        self._rbw_label.show()
+        self._span_label.show()
+        self._diff_label.show()
+        self._mask_label.show()
+        for m in self.marker_labels:
+            m.show()
 
-        arrow_dict = {
-            '32': 'SPACE',
-            '16777235': 'UP KEY',
-            '16777237': 'DOWN KEY',
-            '16777234': 'LEFT KEY',
-            '16777236': 'RIGHT KEY',
-            }
+    def hide_labels(self):
+        self._rbw_label.hide()
+        self._span_label.hide()
+        self._diff_label.hide()
+        self._mask_label.hide()
+        for m in self.marker_labels:
+            m.hide()
 
-        if str(event.key()) in arrow_dict:
-            hotkey = arrow_dict[str(event.key())]
+    def update_rbw_label(self):
+        rfe_mode = self.gui_state.rfe_mode()
+        if rfe_mode == 'HDR':
+            self._rbw_label.setText("RBW:\n%0.2f Hz" % (self.gui_state.rbw))
         else:
-            hotkey = str(event.text()).upper()
-        if hotkey in hotkey_dict:
-            hotkey_dict[hotkey]()
+            self._rbw_label.setText("RBW:\n%0.2f KHz" % (self.gui_state.rbw / 1e3))
 
-
-    def mousePressEvent(self, event):
-        if not self.controller._dut:
-            return
-
-        for marker in self._plot.markers:
-            if marker.selected:
-                break
+    def update_span_label(self):
+        rfe_mode = self.gui_state.rfe_mode()
+        if rfe_mode == 'HDR':
+            self._span_label.setText("SPAN:\n%0.2f KHz" % (self.gui_state.span / 1e3))
         else:
-            return
-
-        trace = self._plot.traces[marker.trace_index]
-        if event.button() == QtCore.Qt.MouseButton.LeftButton:
-            click_pos =  event.pos().x() - 68  # FIXME: declare this as a constant?
-            plot_window_width = self._plot.window.width() - 68
-
-            if click_pos < plot_window_width and click_pos > 0:
-                window_freq = self._plot.view_box.viewRange()[0]
-                window_bw =  (window_freq[1] - window_freq[0])
-                click_freq = ((float(click_pos) / float(plot_window_width)) * float(window_bw)) + window_freq[0]
-                index = find_nearest_index(click_freq, trace.freq_range)
-                marker.data_index = index
+            self._span_label.setText("SPAN:\n%0.2f MHz" % (self.gui_state.span/ M))
 
     def initUI(self):
         grid = QtGui.QGridLayout()
         grid.setSpacing(10)
-        self.plot_width = 8
+        self.plot_width = 11
 
         for x in range(self.plot_width):
             grid.setColumnMinimumWidth(x, 300)
 
-        grid.addWidget(self._plot_layout(),0,0,13,self.plot_width)
+        self._mask_label = QtGui.QLabel()
+        self._mask_label.setStyleSheet('background-color: black')
 
         self.marker_labels = []
-        marker_label, delta_label, diff_label = self._marker_labels()
-        self.marker_labels.append(marker_label)
-        self.marker_labels.append(delta_label)
-        grid.addWidget(marker_label, 0, 1, 1, 2)
-        grid.addWidget(delta_label, 0, 3, 1, 2)
-        grid.addWidget(diff_label , 0, 5, 1, 2)
+        marker_label, delta_label, diff_label, rbw_label, span_label = self._marker_labels()
 
+        grid.addWidget(self._mask_label, 0, 0, 2, self.plot_width)
+        grid.addWidget(marker_label, 0, 3, 1, 2)
+        grid.addWidget(delta_label, 0, 5, 1, 2)
+        grid.addWidget(diff_label , 0, 7, 1, 2)
+        grid.addWidget(self._rbw_label, 0, 0, 1, 2)
+        grid.addWidget(self._span_label, 0, 9, 1, 2)
+        grid.addWidget(self._plot_layout(), 1, 0, 14, self.plot_width)
         y = 0
         x = self.plot_width
         controls_layout = QtGui.QVBoxLayout()
@@ -394,7 +393,7 @@ class MainPanel(QtGui.QWidget):
         controls_layout.addWidget(self._device_controls())
         controls_layout.addWidget(self._trace_controls())
         controls_layout.addStretch()
-        grid.addLayout(controls_layout, y, x, 13, 5)
+        grid.addLayout(controls_layout, y, x, 14, 5)
 
         self._grid = grid
         self.setLayout(grid)
@@ -405,6 +404,15 @@ class MainPanel(QtGui.QWidget):
         vsplit.addWidget(self._plot.window)
         if self._plot.waterfall_window:
             vsplit.addWidget(self._plot.waterfall_window)
+        
+        persist = QtGui.QHBoxLayout()
+        # FIXME: reaching too far into plot..
+        persist.addWidget(self._plot.persistence_window.gradient_editor)
+        persist.addWidget(self._plot.persistence_window)
+        persist_widget = QtGui.QWidget()
+        persist_widget.setLayout(persist)
+        self.persist_widget = persist_widget
+        vsplit.addWidget(persist_widget)
 
         hsplit = QtGui.QSplitter()
         hsplit.addWidget(self._plot.const_window)
@@ -441,27 +449,31 @@ class MainPanel(QtGui.QWidget):
         self.control_widgets.append(self._dev_group)
         return self._dev_group
 
-
-
     def _marker_labels(self):
-        marker_alignment = QtCore.Qt.AlignHCenter
+
         marker_label = QtGui.QLabel('')
-        marker_label.setStyleSheet('color: %s; background-color: black' % colors.TEAL)
-        marker_label.setMinimumHeight(25)
-        marker_label.setAlignment(marker_alignment)
+        marker_label.setAlignment(QtCore.Qt.AlignLeft)
 
         delta_label = QtGui.QLabel('')
-        delta_label.setStyleSheet('color: %s;' % colors.TEAL)
-        delta_label.setMinimumHeight(25)
-        delta_label.setAlignment(marker_alignment)
+        delta_label.setAlignment(QtCore.Qt.AlignLeft)
+
+        span_label = QtGui.QLabel('')
+        span_label.setStyleSheet(fonts.MARKER_LABEL_FONT % (colors.BLACK_NUM + colors.GREY_NUM))
+        span_label.setAlignment(QtCore.Qt.AlignLeft)
+
+        rbw_label = QtGui.QLabel('')
+        rbw_label.setStyleSheet(fonts.MARKER_LABEL_FONT % (colors.BLACK_NUM + colors.GREY_NUM))
+        rbw_label.setAlignment(QtCore.Qt.AlignRight)
 
         diff_label = QtGui.QLabel('')
-        diff_label.setStyleSheet('color: %s;' % colors.WHITE)
-        diff_label.setMinimumHeight(25)
-        diff_label.setAlignment(marker_alignment)
-        self._diff_lab = diff_label
+        diff_label.setAlignment(QtCore.Qt.AlignLeft)
+        self._diff_label = diff_label
+        self._rbw_label = rbw_label
+        self._span_label = span_label
+        self.marker_labels.append(marker_label)
 
-        return marker_label,delta_label, diff_label
+        self.marker_labels.append(delta_label)
+        return marker_label,delta_label, diff_label, rbw_label, span_label
 
     def capture_received(self, state, fstart, fstop, raw, power, usable, segments):
         """
@@ -473,6 +485,7 @@ class MainPanel(QtGui.QWidget):
         :param usable: usable bins from power (None when sweeping)
         :param segments: bin segments from power (None when not sweeping)
         """
+
         self.raw_data = raw
         self.pow_data = power
         self.usable_bins = usable
@@ -493,7 +506,7 @@ class MainPanel(QtGui.QWidget):
         if self.iq_plots_enabled:
             self.update_iq()
 
-        if self.waterfall_plot_enabled:
+        if self.waterfall_plot_enabled or self.persistence_plot_enabled:
             if (fstart, fstop, len(power)) != self._waterfall_range:
                 self._plot.waterfall_data.reset(self.xdata)
                 self._waterfall_range = (fstart, fstop, len(power))
@@ -503,17 +516,21 @@ class MainPanel(QtGui.QWidget):
     def options_changed(self, options, changed):
         self.iq_plots_enabled = options['iq_plots']
         self.waterfall_plot_enabled = options['waterfall_plot']
+        self.persistence_plot_enabled = options['persistence_plot']
 
         if 'iq_plots' in changed:
             self._update_iq_plot_visibility()
 
         ww = self._plot.waterfall_window
         if 'waterfall_plot' in changed and ww:
-            if options['waterfall_plot']:
+            if self.waterfall_plot_enabled:
                 ww.show()
             else:
                 ww.hide()
 
+        if 'persistence_plot' in changed:
+            self.persist_widget.setVisible(self.persistence_plot_enabled)
+
     def _update_iq_plot_visibility(self):
         if not self.gui_state:
             return
@@ -587,18 +604,18 @@ class MainPanel(QtGui.QWidget):
                     trace = self._plot.traces[marker.trace_index]
                     marker_label.show()
                     if not trace.blank:
-                        if marker.selected:
-                            color = colors.YELLOW_NUM
-                        else:
-                            color = colors.WHITE_NUM
-                        marker_label.setStyleSheet(fonts.MARKER_LABEL_FONT % (colors.BLACK_NUM + color))
 
+                        marker_label.setStyleSheet(fonts.MARKER_LABEL_FONT % (colors.BLACK_NUM + marker.draw_color))
                         marker.update_pos(trace.freq_range, trace.data)
-                        marker_text = 'M%d: %0.2f MHz \n %0.2f dBm' % (num, trace.freq_range[marker.data_index]/1e6, 
+                        if self.gui_state.rfe_mode() == 'HDR':
+                            marker_text = 'M%d: %0.4f MHz \n %0.2f dBm' % (num, trace.freq_range[marker.data_index]/1e6, 
+                                                                           trace.data[marker.data_index])
+                        else:
+                            marker_text = 'M%d: %0.2f MHz \n %0.2f dBm' % (num, trace.freq_range[marker.data_index]/1e6, 
                                                                                    trace.data[marker.data_index])
                         num += 1
                         marker_label.setText(marker_text)
-
+                        self._mask_label.show()
                 else:
                     marker_label.hide()
 
@@ -613,17 +630,22 @@ class MainPanel(QtGui.QWidget):
                 num_markers += 1
                 traces.append(self._plot.traces[marker.trace_index])
                 data_indices.append(marker.data_index)
-                
-        if num_markers == len(labels.MARKERS):
-            freq_diff = np.abs((traces[0].freq_range[data_indices[0]]/1e6) - (traces[1].freq_range[data_indices[1]]/1e6))
-            
-            power_diff = np.abs((traces[0].data[data_indices[0]]) - (traces[1].data[data_indices[1]]))
-            self._diff_lab.setStyleSheet(fonts.MARKER_LABEL_FONT % (colors.BLACK_NUM + colors.WHITE_NUM))
-            delta_text = 'Delta: %0.1f MHz \n %0.2f dB' % (freq_diff, power_diff )
-            self._diff_lab.setText(delta_text)
-        else:
-            self._diff_lab.setText('')
 
+        if num_markers == len(labels.MARKERS):
+
+            self._diff_label.show()
+
+            freq_diff = np.abs((traces[0].freq_range[data_indices[0]]/1e6) - (traces[1].freq_range[data_indices[1]]/1e6))
+
+            power_diff = np.abs((traces[0].data[data_indices[0]]) - (traces[1].data[data_indices[1]]))
+            self._diff_label.setStyleSheet(fonts.MARKER_LABEL_FONT % (colors.BLACK_NUM + colors.GREY_NUM))
+            if self.gui_state.rfe_mode() == 'HDR':
+                delta_text = 'Delta: %0.2f KHz \n %0.2f dB' % (freq_diff * 1000, power_diff )
+            else:
+                delta_text = 'Delta: %0.1f MHz \n %0.2f dB' % (freq_diff, power_diff )
+            self._diff_label.setText(delta_text)
+        else:
+            self._diff_label.hide()
     def enable_controls(self):
         for item in self.control_widgets:
             item.setEnabled(True)
