@@ -179,8 +179,10 @@ class TraceControls(QtGui.QGroupBox):
             m = 0 if not self._plot.markers[0].enabled else 1
             self._plot.markers[m].enable(self._plot)
             self._plot.markers[m].trace_index = num
-            if not self._markers[m].marker.isChecked():
-                self._markers[m].marker.click()  # select markers when adding
+            if m == 0:
+                self.controller.apply_plot_options(marker0 = True)
+            else:
+                self.controller.apply_plot_options(marker1 = True)
             self._build_layout()
         add_marker.clicked.connect(add_marker_clicked)
 
@@ -196,12 +198,7 @@ class TraceControls(QtGui.QGroupBox):
         :returns: a MarkerWidgets namedtuple
 
         """
-        radio = QtGui.QRadioButton("M%d:" % (num + 1))
-        button_group.addButton(radio)
-        def marker_select():
-            for i, marker in enumerate(self._plot.markers):
-                marker.selected = i == num
-        radio.clicked.connect(marker_select)
+        label = QtGui.QLabel("M%d:" % (num + 1))
 
         center = QtGui.QPushButton("Center")
         center.setToolTip("Center the frequency on this marker")
@@ -231,20 +228,20 @@ class TraceControls(QtGui.QGroupBox):
             self._find_right_peak(num)
         peak_right.clicked.connect(peak_right_clicked)
 
-        remove_marker = QtGui.QPushButton("-")
+        remove_marker = QtGui.QPushButton("- Marker")
         remove_marker.setMinimumWidth(REMOVE_BUTTON_WIDTH)
         remove_marker.setToolTip("Remove this marker")
         def remove_marker_clicked():
-            if self._markers[num].marker.isChecked():
-                for m in self._markers:
-                    if not m.marker.isChecked():
-                        break
-                m.marker.click()  # select other marker
+            alt_mark = not num
+            if num == 0:
+                self.controller.apply_plot_options(marker0 = False)
+            else:
+                self.controller.apply_plot_options(marker1 = False)
             self._plot.markers[num].disable(self._plot)
             self._build_layout()
         remove_marker.clicked.connect(remove_marker_clicked)
 
-        return MarkerWidgets(radio, center, peak_left, peak, peak_right,
+        return MarkerWidgets(label, center, peak_left, peak, peak_right,
             remove_marker)
 
     def _build_layout(self):
@@ -283,12 +280,12 @@ class TraceControls(QtGui.QGroupBox):
 
         def add_marker_widgets(marker_widgets, row):
             show(marker_widgets.marker, row, 0, 3,3)
-            show(marker_widgets.peak, row, 2, 1, 2)
-            show(marker_widgets.center, row, 4, 1, 2)
-            show(marker_widgets.remove, row, 6, 1, 1)
+            show(marker_widgets.peak, row, 1, 1, 2)
+            show(marker_widgets.center, row, 3, 1, 2)
+            show(marker_widgets.remove, row, 5, 1, 2)
             row = row + 1
-            show(marker_widgets.peak_left, row, 2, 1, 2)
-            show(marker_widgets.peak_right, row, 4, 1, 2)
+            show(marker_widgets.peak_left, row, 1, 1, 2)
+            show(marker_widgets.peak_right, row, 3, 1, 2)
             return row + 1
 
         extra_markers = any(not m.enabled for m in self._plot.markers)
@@ -345,60 +342,6 @@ class TraceControls(QtGui.QGroupBox):
         """
         self._plot.traces[num].store = bool(store)
 
-    def _marker_control(self):
-        """
-        disable/enable marker
-        """
-        marker = self._plot.markers[self._marker_tab.currentIndex()]
-        if self._marker_check.checkState() is QtCore.Qt.CheckState.Checked:
-
-            self._marker_trace.setEnabled(True)
-            if self._marker_trace.currentIndex() < 0:
-                self._marker_trace.setCurrentIndex(0)
-            marker.trace_index = int(self._marker_trace.currentText()) - 1
-            marker.enable(self._plot)
-        else:
-            self._marker_trace.setEnabled(False)
-            self._plot.markers[self._marker_tab.currentIndex()].disable(self._plot)
-
-            self.marker_labels[self._marker_tab.currentIndex()].setText('')
-
-    def _marker_trace_control(self):
-        """
-        change the trace that is currently associated with the marker
-        """
-
-        if self._marker_trace is not None:
-            marker = self._plot.markers[0]  # XXX
-            if not self._marker_trace.currentText() == '':
-                marker.trace_index = int(self._marker_trace.currentText()) - 1
-
-    def _marker_tab_change(self):
-        """
-        change the current selected marker
-        """
-
-        for marker in self._plot.markers:
-            marker.selected = False
-        marker = self._plot.markers[self._marker_tab.currentIndex()]
-        if marker.enabled:
-            if marker.trace_index == 2:
-                if self._marker_trace.count() == 2:
-                    index = 1
-                else:
-                    index = 2
-                self._marker_trace.setCurrentIndex(index)
-            else:
-                self._marker_trace.setCurrentIndex(marker.trace_index)
-            self._marker_trace.setEnabled(True)
-            self._marker_check.setCheckState(QtCore.Qt.CheckState.Checked)
-        else:
-            self._marker_trace.setEnabled(False)
-
-            self._marker_trace.setCurrentIndex(marker.trace_index)
-            self._marker_check.setCheckState(QtCore.Qt.CheckState.Unchecked)
-        marker.selected = True
-
     def _find_peak(self, num):
         """
         move the selected marker to the maximum point of the spectrum
@@ -481,81 +424,3 @@ class TraceControls(QtGui.QGroupBox):
             return
         marker.data_index = np.where(pow_data==(peak_values[-2 if len(peak_values) > 1 else -1]))[0]
 
-
-    def plot_controls(self):
-
-        plot_group = QtGui.QGroupBox("Amplitude Control")
-        plot_group.setStyleSheet(GROUP_BOX_FONT)
-        self._plot_group = plot_group
-
-        grid = QtGui.QGridLayout()
-
-        self.control_widgets = []
-
-        ref_level, ref_label, min_level, min_label = self._ref_controls()
-
-        grid.addWidget(ref_label, 0, 0, 1, 1)
-        grid.addWidget(ref_level, 0, 1, 1, 1)
-        grid.addWidget(min_label, 0, 3, 1, 1)
-        grid.addWidget(min_level, 0, 4, 1, 1)
-
-        grid.setColumnStretch(0, 3)
-        grid.setColumnStretch(1, 6)
-        grid.setColumnStretch(2, 1)
-        grid.setColumnStretch(3, 4)
-        grid.setColumnStretch(4, 6)
-
-        plot_group.setLayout(grid)
-
-        return plot_group
-
-    def _center_control(self):
-        center = QtGui.QPushButton('Recenter')
-        center.setToolTip("[C]\nCenter the Plot View around the available spectrum")
-        center.clicked.connect(lambda: self._plot.center_view(min(self.xdata),
-                                                            max(self.xdata),
-                                                            min_level = int(self._min_level.text()),
-                                                            ref_level = int(self._ref_level.text())))
-        self._center_bt = center
-        self.control_widgets.append(self._center_bt)
-        return center
-
-
-    def _update_plot_y_axis(self):
-        min_level = self._min_level.value()
-        ref_level = self._ref_level.value()
-        
-        self._plot.center_view(
-            self.xdata[0], self.xdata[-1],
-            min_level = min_level,
-            ref_level = ref_level)
-        
-        self._plot.update_waterfall_levels(min_level, ref_level)
-
-    def _ref_controls(self):
-        ref_level = QtGui.QSpinBox()
-        ref_level.setRange(PLOT_YMIN, PLOT_YMAX)
-        ref_level.setValue(PLOT_TOP)
-        ref_level.setSuffix(" dBm")
-        ref_level.setSingleStep(PLOT_STEP)
-        ref_level.valueChanged.connect(self._update_plot_y_axis)
-        self._ref_level = ref_level
-        self.control_widgets.append(self._ref_level)
-        ref_label = QtGui.QLabel('Ref Level: ')
-
-        min_level = QtGui.QSpinBox()
-        min_level.setRange(PLOT_YMIN, PLOT_YMAX)
-        min_level.setValue(PLOT_BOTTOM)
-        min_level.setSuffix(" dBm")
-        min_level.setSingleStep(PLOT_STEP)
-        min_level.valueChanged.connect(self._update_plot_y_axis)
-        min_label = QtGui.QLabel('Min Level: ')
-        self._min_level = min_level
-        self.control_widgets.append(self._min_level)
-        return ref_level, ref_label, min_level, min_label
-
-    def get_ref_level(self):
-        return self._ref_level.value()
-    def get_min_level(self):
-
-        return self._min_level.value()
